@@ -1,7 +1,7 @@
 package App::rimetadb;
 
-our $DATE = '2016-09-27'; # DATE
-our $VERSION = '0.18'; # VERSION
+our $DATE = '2016-12-10'; # DATE
+our $VERSION = '0.19'; # VERSION
 
 use 5.010001;
 use strict;
@@ -142,10 +142,27 @@ our %args_query = (
     query => {
         schema => 'str*',
         pos => 0,
+        tags => ['category:filtering'],
     },
     %args_query_detail,
 );
 
+our %args_package = (
+    package => {
+        summary => 'Select specific package only',
+        schema => 'str*', # XXX package name
+        tags => ['category:filtering'],
+    },
+);
+
+our %args_function = (
+    function => {
+        summary => 'Select specific function only',
+        schema => 'str*', # XXX function name
+        tags => ['category:filtering'],
+    },
+);
+
 sub _is_excluded {
     my ($x, $exc_list) = @_;
     for (@$exc_list) {
@@ -649,6 +666,7 @@ $SPEC{functions} = {
     args => {
         %args_common,
         %args_query,
+        %args_package,
     },
 };
 sub functions {
@@ -669,6 +687,11 @@ sub functions {
         push @binds, $q, $q, $q, $q;
     }
 
+    if (defined $args{package}) {
+        push @wheres, "(package=?)";
+        push @binds, $args{package};
+    }
+
     my $sth = $dbh->prepare(
         "SELECT package,name,summary,dist,mtime,extra FROM function ORDER by package,name".
             (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
@@ -686,6 +709,77 @@ sub functions {
     [200, "OK", \@rows, {'table.fields'=>\@columns}];
 }
 
+$SPEC{arguments} = {
+    v => 1.1,
+    summary => 'List function arguments',
+    args => {
+        %args_common,
+        %args_query,
+        %args_package,
+        %args_function,
+    },
+};
+sub arguments {
+    require Data::Sah::Util::Type;
+
+    my %args = @_;
+
+    my ($res, $dbh) = _connect_db(\%args);
+    return $res unless $res->[0] == 200;
+
+    my $q  = $args{query};
+
+    my @rows;
+    my @columns = qw(name package function summary schema schema_type req pos greedy);
+    my @wheres;
+    my @binds;
+
+    if (length $q) {
+        push @wheres, "(package LIKE ? OR name LIKE ? OR dist LIKE ? OR extra LIKE ?)";
+        push @binds, $q, $q, $q, $q;
+    }
+
+    if (defined $args{package}) {
+        push @wheres, "(package=?)";
+        push @binds, $args{package};
+    }
+
+    if (defined $args{function}) {
+        push @wheres, "(function=?)";
+        push @binds, $args{function};
+    }
+
+    my $sth = $dbh->prepare(
+        "SELECT package,name AS function,metadata FROM function ORDER by package,name".
+            (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
+    );
+    $sth->execute(@binds);
+
+    while (my $row0 = $sth->fetchrow_hashref) {
+        my $meta = _json->decode(delete $row0->{metadata});
+        if ($meta->{args}) {
+            for my $arg (sort keys %{ $meta->{args} }) {
+                my $argspec = $meta->{args}{$arg};
+                my $row = {%$row0};
+                $row->{name} = $arg;
+                $row->{summary} = $argspec->{summary};
+                $row->{schema} = _json->encode($argspec->{schema});
+                $row->{schema_type} = Data::Sah::Util::Type::get_type($argspec->{schema});
+                $row->{req} = $argspec->{req};
+                $row->{pos} = $argspec->{pos};
+                $row->{greedy} = $argspec->{greedy};
+                if ($args{detail}) {
+                    push @rows, $row;
+                } else {
+                    push @rows, "$row->{package}\::$row->{function}\::$row->{name}";
+                }
+            }
+        }
+    }
+
+    [200, "OK", \@rows, {'table.fields'=>\@columns}];
+}
+
 $SPEC{stats} = {
     v => 1.1,
     summary => 'Show some statistics',
@@ -710,16 +804,16 @@ sub stats {
     [200, "OK", \%stats];
 }
 
-$SPEC{arguments} = {
+$SPEC{argument_stats} = {
     v => 1.1,
-    summary => 'List function arguments',
+    summary => 'Show statistics on function arguments',
     args => {
         %args_common,
         #%args_query,
-        %args_query_detail,
+        #%args_query_detail,
     },
 };
-sub arguments {
+sub argument_stats {
     my %args = @_;
 
     my ($res, $dbh) = _connect_db(\%args);
@@ -756,9 +850,9 @@ sub arguments {
         push @rows, {name=>$_, num_occurences=>$num_occurences{$_}};
     }
 
-    unless ($args{detail}) {
-        @rows = map {$_->{name}} @rows;
-    }
+    #unless ($args{detail}) {
+    #    @rows = map {$_->{name}} @rows;
+    #}
 
     my @columns = qw(name num_occurences);
     [200, "OK", \@rows, {'table.fields'=>\@columns}];
@@ -822,7 +916,7 @@ App::rimetadb - Manage a Rinci metadata database
 
 =head1 VERSION
 
-This document describes version 0.18 of App::rimetadb (from Perl distribution App-rimetadb), released on 2016-09-27.
+This document describes version 0.19 of App::rimetadb (from Perl distribution App-rimetadb), released on 2016-12-10.
 
 =head1 SYNOPSIS
 
@@ -831,6 +925,47 @@ See the included CLI script L<rimetadb>.
 =head1 FUNCTIONS
 
 
+=head2 argument_stats(%args) -> [status, msg, result, meta]
+
+Show statistics on function arguments.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<dsn> => I<str>
+
+DBI connection DSN.
+
+If not specified, will default to C<dbd:SQLite:$HOME/rimeta.db> where C<$HOME> is
+user's home directory.
+
+Note: has been tested with MySQL and SQLite only.
+
+=item * B<password> => I<str>
+
+DBI connection password.
+
+=item * B<user> => I<str>
+
+DBI connection user.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 arguments(%args) -> [status, msg, result, meta]
 
 List function arguments.
@@ -852,10 +987,20 @@ user's home directory.
 
 Note: has been tested with MySQL and SQLite only.
 
+=item * B<function> => I<str>
+
+Select specific function only.
+
+=item * B<package> => I<str>
+
+Select specific package only.
+
 =item * B<password> => I<str>
 
 DBI connection password.
 
+=item * B<query> => I<str>
+
 =item * B<user> => I<str>
 
 DBI connection user.
@@ -940,6 +1085,10 @@ user's home directory.
 
 Note: has been tested with MySQL and SQLite only.
 
+=item * B<package> => I<str>
+
+Select specific package only.
+
 =item * B<password> => I<str>
 
 DBI connection password.
@@ -1240,7 +1389,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/App-rimeta
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-App-rimetadb>.
+Source repository is at L<https://github.com/perlancar/perl-App-UpdateRinciMetadataDb>.
 
 =head1 BUGS
 
